'''there are 7 modules in this package you have teh DataBase class, DataDiscovery,
DataVisulization, DataCorrelation, Encoder, MissingValues, FeatureScaler'''
from .datadiscovery import DataDiscovery
from .datacorrelation import DataCorrelation
from .encoder2 import Encoder
from .missingvalues import MissingValues
from .featur_scaling import FeatureScaler
from .inconsistent_data import InconsistentData
from .stringtodatetime import StringToDateTime
from .resource import Resources



import pandas as pd
from sklearn.preprocessing import OrdinalEncoder
from .stringtodatetime import StringToDateTime
from .inconsistent_data import InconsistentData
class Encoder:
    def __init__(self,df,type = "", columns = []):
        ''' this class is dessigned to help you make encoding your data simple
        the input variables for this class are
        df: a pandas dataframe
        type: by defalult this variable will br set to ONEHOTENCODER if you with to use
        OrdinalEncoder you would set type to ordinalencoder
        then you can call the check method to make the corretions
        this method will return a pandas data frame
        if you wish to compare the returned value to the original dataset you may
        call copy'''
        self.df = df
        self.copy = df.copy()
        self.type = type
        self.columns = columns
        datetime = StringToDateTime(self.df)
        self.df = datetime.check()
        correcting = InconsistentData(self.df)
        correcting.column_names_white_space()
        self.df = correcting.data_white_space()

    def check(self):
        if (len(self.columns)==0):
            self.object_column = []
            for i in self.df.columns:
                if (self.df[i].dtype == "object"):
                    self.object_column.append(i)
        else:
            self.object_column = [i for i in self.columns]
        self.Correct()
        return self.df
    def Correct(self):
        if (self.type == "" or self.type.upper() == "ONEHOTENCODER"):
            self.OneHotEncoder()
        elif (self.type.upper() == "ORDINALENCODER"):
            self.OrdinalEncoder()
    def OrdinalEncoder(self):
        for i in self.object_column:
            translate = OrdinalEncoder()
            final = translate.fit_transform(self.df[i].array.reshape(-1, 1))
            self.df.drop(columns=i, inplace=True)
            self.df[i] = final

    def OneHotEncoder(self):
        for i in self.object_column:
            new_df = pd.get_dummies(self.df[i])
            self.new_df = pd.concat([self.df,new_df],axis=1)
        self.new_df.drop(columns=self.object_column,inplace=True)
        self.df = self.new_df



if __name__ == "__main__":
    import pandas as pd

    data = pd.read_csv("/Users/williammckeon/Sync/youtube videos/novembers 2021/Parsing data/code/travel_times.csv")
    # data = pd.read_csv('https://raw.githubusercontent.com/jldbc/coffee-quality-database/master/data/arabica_data_cleaned.csv')
    # data = pd.read_csv("travel_times.csv")
    print(type(data))
    stringtodate = Encoder(df=data)
    data = stringtodate.check()
    print(data.dtypes)
    # print(stringtodate.new_df.columns)



from encoder2 import Encoder
import numpy as np
import pandas as pd
from sklearn.impute import SimpleImputer
class MissingValues:
    '''MissingValues is dessigned to make correcting missing values alot more accesable.
    MissingValues(df)
    df: is the inputted pandas dataframe what will have corrections made to it
    check is the method used to tell the module to start the corrections, this method will return the corrected dataframe
    if you wish to get the original dataframe call the copy variable.
    currently you are only able to use the median stratagy however other methods are in the work'''
    def __init__(self, df, strategy = ["median"], columns = [], columns_drop = [],
                 columns_fill =[],whole_df = False, fill_with = 0, round = True):
        # data = Encoder(df = df)
        # self.df = data.check()
        self.df = df
        # print(self.df.dtypes)
        self.strategy = strategy
        self.columns = columns
        self.whole_df = whole_df
        self.np_df = self.df.iloc[:,:].values
        self.fill_with = fill_with
        self.column_location = {}
        self.round = round
        self.columns_drop = columns_drop
        self.columns_fill = columns_fill
    def check(self):
        self.column_location = {}
        count = 0
        for i in self.columns:
            self.column_location[i] = count
            count+=1
        for i in self.strategy:
            if i.upper()=="MEDIAN":
                self.median()
                # self.df = pd.DataFrame(self.np_df, columns=self.df.columns)
            elif i.upper()=="FILL":
                self.df.fillna(self.fill_with, inplace = True)
            elif i.upper()=="DROP":
                self.df.dropna(inplace = 0)
        return self.df
    def median(self):
        for i in self.columns:
            for j in range(len(self.np_df[:][self.column_location[i]])):
                print(j)
                if np.isnan(self.np_df[j][self.column_location[i]]):
                    self.np_df[j][self.column_location[i]] = self.np_df[j][self.column_location[i]].mean()
                    # j = round(self.np_df[j][self.column_location[i]].mean())
        # self.isnull = []
        # for i in self.df.columns:
        #     if (self.df[i].isnull().values.any()):
        #         self.isnull.append(i)
        # if (len(self.isnull) >0):
        #     self.Imputer()
        # return self.df
    # def Imputer(self):
    #     imputer = SimpleImputer(strategy="median")
    #     new_data = imputer.fit_transform(self.df)
    #     convert = pd.DataFrame(new_data, columns=self.df.columns,index=self.df.index)
    #     self.df = convert
if __name__ == "__main__":
    import pandas as pd
    from stringtodatetime import StringToDateTime
    data = pd.read_csv("/Users/williammckeon/Sync/youtube videos/novembers 2021/Parsing data/code/travel_times.csv",index_col=0)
    # data = pd.read_csv('https://raw.githubusercontent.com/jldbc/coffee-quality-database/master/data/arabica_data_cleaned.csv')
    # data = pd.read_csv("travel_times.csv")
    # print(type(data))
    print(data.dtypes)
    stringtodate = StringToDateTime(data)
    data = stringtodate.check()
    print(data.info())
    missing = MissingValues(data, columns=["AvgSpeed","FuelEconomy"])
    data = missing.check()

    # for i in data.columns:
    #     print(data[i].count())
    # # print(stringtodate.new_df.columns)


import pandas as pd
import datetime

class StringToDateTime:
    ''' this class is designed to make converting strings to datetime more accessable
    this is done by creating an instance of the class StringToDateTime(df, column_names)
    df is where you will define the pandas dataframe that you will work with
    column_names is when you have a column names for columns you wish have converted to datetime
    that is not not ["date","dates","starttime","start_time","start time"], to use this input argument
    successfully you must pass in a list
    check(): to tell the module to make the corrections you must call the check method
    resources(): will give you the link to the youtube video about this module as well as the github
    '''

    def __init__(self,df, column_names = []):
        if isinstance(column_names,list) is False:
            raise AttributeError("column_names needs to be a list")
        self.df = df
        self.copy_df = df.copy()
        self.time_spot = {}
        self.time_list = ["date","dates","starttime","start_time","start time"]
        self.possible = [" " ,",","/","-", ":"]
        self.greates = [i*0 for i in range(len(self.possible))]
        if (len(column_names)>0):
            for i in column_names:
                self.time_list.append(i)

    def _finding_the_columns(self):
        count = 0
        # time_spot = {}
        for i in self.df.columns:
            if i.lower() in set(self.time_list):
                self.time_spot[i] = count
            count+=1

    def _making_the_changes(self):
        for j in self.time_spot.keys():
            for i in self.df[j]:
                self.greates[0] = self.greates[0]+i.count(self.possible[0])
                self.greates[1] = self.greates[1]+i.count(self.possible[1])
                self.greates[2] = self.greates[2]+i.count(self.possible[2])
                self.greates[3] = self.greates[3]+i.count(self.possible[3])
                self.greates[4] = self.greates[4]+i.count(self.possible[4])
            for i in range(len(self.greates)):
                if self.greates[i] == max(self.greates):
                    most_used_char = self.possible[i]
            for w in self.df[j]:
                rep = ""
                for l in w:
                    try:
                        int(l)
                        rep = rep+l
                    except ValueError:
                        rep =rep+most_used_char

                self.df[j].replace(to_replace={w:rep}, inplace=True)
        # data["Date"] = pd.to_datetime(data["Date"],infer_datetime_format=True)
            self.df[j] = pd.to_datetime(self.df[j],infer_datetime_format=True)
            self.greates = [i*0 for i in range(len(self.possible))]
    def check(self):
        self._finding_the_columns()
        self._making_the_changes()
        return self.df
    def resource(self):
        links ={}
        links["youtube"] = "https://youtu.be/j5m9UgnP5bU"
        links["github"] = "https://github.com/Islanderrobotics/string_to_date_time"
        for i in links.keys():
            print(f"the link for {i} is {links[i]}")

if __name__ =="__main__":
    import pandas as pd
    import datetime
    from stringtodatetime import StringToDateTime
    # data = pd.read_csv("landslide_data3.csv")
    data = pd.read_csv('https://raw.githubusercontent.com/jldbc/coffee-quality-database/master/data/arabica_data_cleaned.csv')
# data = pd.read_csv("travel_times.csv"

    stringtodate = StringToDateTime(df = data)
    data = stringtodate.check()
    stringtodate.resource()